from mock import MagicMock, patch

from redberry.tests import RedTestCase
from redberry.models import RedPost, RedCategory


class AdminRoutesTest(RedTestCase):

    @patch('redberry.blueprint.current_user')
    def test_admin_defaults_to_categories(self, mock_user):
        url = "/%s/admin" % self.url_prefix
        categories = RedCategory.query.all()

        response = self.test_client.get(url)
        for category in categories:
            assert category.title in response.data

    @patch('redberry.blueprint.current_user')
    def test_admin_shows_posts(self, mock_user):
        url = "/%s/admin/post" % self.url_prefix
        categories = RedCategory.query.all()

        response = self.test_client.get(url)
        for category in categories:
            assert category.title not in response.data

        posts = RedPost.query.all()
        for post in posts:
            assert post.title in response.data

    @patch('redberry.blueprint.current_user')
    def test_new_record_creates_category(self, mock_user):
        url = "/%s/admin/category/new" % self.url_prefix
        assert RedCategory.query.count() == 1

        response = self.test_client.post(url, data={'title': "A new category"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin' % self.url_prefix
        self.assert_flashes("Saved category 2")
        assert RedCategory.query.count() == 2

        new_category = RedCategory.query.all()[-1]
        assert new_category.title == "A new category"

    @patch('redberry.blueprint.current_user')
    def test_new_record_creates_post(self, mock_user):
        url = "/%s/admin/post/new" % self.url_prefix
        assert RedPost.query.count() == 1

        response = self.test_client.post(url, data={'title': "A new post"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin/post' % self.url_prefix
        self.assert_flashes("Saved post 2")
        assert RedPost.query.count() == 2

        new_post = RedPost.query.all()[-1]
        assert new_post.title == "A new post"

    @patch('redberry.blueprint.current_user')
    def test_edit_record_updates_category(self, mock_user):
        category = RedCategory.query.first()
        url = "/%s/admin/category/%s" % (self.url_prefix, category.slug)
        assert RedCategory.query.count() == 1

        response = self.test_client.post(url, data={'title': "A new title"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin' % self.url_prefix
        self.assert_flashes("Saved category 1")
        assert RedCategory.query.count() == 1
        assert category.title == "A new title"

    @patch('redberry.blueprint.current_user')
    def test_edit_record_updates_post(self, mock_user):
        post = RedPost.query.first()
        url = "/%s/admin/post/%s" % (self.url_prefix, post.slug)
        assert RedPost.query.count() == 1

        response = self.test_client.post(url, data={'title': "A new title"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin/post' % self.url_prefix
        self.assert_flashes("Saved post 1")
        assert RedPost.query.count() == 1
        assert post.title == "A new title"

    @patch('redberry.blueprint.current_user')
    def test_edit_record_deletes_category(self, mock_user):
        category = RedCategory.query.first()
        url = "/%s/admin/category/%s" % (self.url_prefix, category.slug)
        assert RedCategory.query.count() == 1

        response = self.test_client.post(url, data={'_method': "DELETE"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin' % self.url_prefix
        self.assert_flashes("Deleted category")
        assert RedCategory.query.count() == 0

    @patch('redberry.blueprint.current_user')
    def test_edit_record_deletes_post(self, mock_user):
        post = RedPost.query.first()
        url = "/%s/admin/post/%s" % (self.url_prefix, post.slug)
        assert RedPost.query.count() == 1

        response = self.test_client.post(url, data={'_method': "DELETE"})
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/admin/post' % self.url_prefix
        self.assert_flashes("Deleted post")
        assert RedPost.query.count() == 0


from slugify import slugify
from html5lib_truncation import truncate_html
from bs4 import BeautifulSoup
import re
from string import punctuation as p

from redberry.blueprint import cms

db = cms.config['db']


def generate_slug(target, value, oldvalue, initiator):
    if value and (not target.slug or value != oldvalue):
        target.slug = slugify(value)


class RedModel(db.Model):
    """
    All Redberry models inherit from this base class. It adds Django-like save() and delete() methods.
    """
    __abstract__ = True

    @staticmethod
    def save():
        db.session.flush()

    def delete(self):
        db.session.delete(self)
        db.session.flush()

    @staticmethod
    def strip_tags(text, strip_punctuation=False):
        # Return only the words from content, stripping punctuation and HTML.
        soup = BeautifulSoup(text)

        if strip_punctuation:
            punctuation = re.compile('[{}]+'.format(re.escape(p)))
            words_only = punctuation.sub('', soup.get_text())
            return words_only

        words_only = soup.get_text()
        return words_only

    def summarise(self, length=150):
        long_field = None

        if hasattr(self, 'summary'):
            long_field = getattr(self, 'summary')

        if not long_field:
            for field in ('content', 'description'):
                if hasattr(self, field):
                    long_field = getattr(self, field)

        if not long_field:
            return ''

        if len(long_field) <= length:
            return long_field

        return truncate_html(long_field, length, end='...')


# Enable all models to be imported from the module. Import at the end to prevent circular imports.
from .category import RedCategory
from .post import RedPost
from .version import RedVersion


import logging
from sqlalchemy.sql.elements import quoted_name
import importlib

from redberry.blueprint import cms
from redberry.models import RedPost, RedCategory, RedVersion
from redberry.models.post import categories_posts


class RedMigrator:

    def __init__(self):
        self.db = cms.config['db']
        self.logger = logging.getLogger('redberry')

    def do_migration(self):
        for model in [RedVersion, RedPost, RedCategory]:
            if not self.db.engine.has_table(quoted_name(model.__table__, True)):
                self.logger.warning("Creating database table %s" % model.__tablename__)
                model.__table__.create(self.db.engine)

        if not self.db.engine.has_table(categories_posts.name):
            self.logger.warning("Creating database join table %s" % categories_posts.name)
            categories_posts.create(self.db.engine)

        # Run migrations
        migrations = RedVersion.all_migrations()
        self.logger.warning("--- Migrations %s" % migrations)

        # Migrations ['201612311100_add_publish_state.py', '201612311300_dummy.py']
        for migration in migrations:
            migration_file = migration.split(".")[0]  # eg "201612311100_add_publish_state"
            self.run_migration(migration_file=migration_file)

    def initialize_samples(self):
        if not RedCategory.query.count():
            self.logger.info("Inserting sample category")
            category = RedCategory(title="Tips & Tricks", description="A few posts for working with Redberry")
            self.db.session.add(category)

        if not RedPost.query.count():
            self.logger.info("Inserting sample post")
            post = RedPost(title="Berries, berries everywhere!", content="A small berry to start the day never hurt anybody.")
            post.categories = [RedCategory.query.first()]
            self.db.session.add(post)

        self.db.session.flush()

    def run_migration(self, migration_file, method='upgrade'):

        if RedVersion.already_run(migration_file):
            self.logger.warn("Migration %s has already been run, skipping." % migration_file)
            return

        try:
            module = importlib.import_module('redberry.models.migrations.%s' % migration_file)

            if method == 'upgrade':
                self.logger.info("Running upgrade on %s" % migration_file)
                module.upgrade(self.db)
            else:
                self.logger.info("Running downgrade on %s" % migration_file)
                module.downgrade(self.db)

            RedVersion.store_migration(migration_file)

        except Exception, e:
            self.logger.error("Error running %s" % migration_file)
            self.logger.error(e)




import os
import functools
from flask import Blueprint, render_template, flash, redirect, url_for, request
from flask.ext.login import current_user

REDBERRY_ROOT = os.path.dirname(os.path.realpath(__file__))

cms = Blueprint('redberry', __name__, template_folder='templates', static_folder='static/redberry')
cms.config = {}


##############
# Jinja Filters
###############
@cms.app_template_filter()
def pretty_date(dttm):
    return dttm.strftime("%m/%d")

@cms.app_template_filter()
def date(dttm):
    return dttm.strftime('%Y-%m-%d')


########
# Config
########
# Automatically called once when the Blueprint is registered on the app.
@cms.record_once
def init_redberry(state):
    config = state.app.config.get("redberry.config")

    if config is None:
        raise Exception("Redberry expects you to provide a dict in the redberry.config setting for your app")

    if 'db' not in config:
        raise Exception("Redberry expects you to provide an SQLAlchemy object in redberry.config['db']")

    cms.config['db'] = config['db']

    from redberry.utils.logger import init_logger
    init_logger()

    from redberry.models.migrate import RedMigrator
    migrator = RedMigrator()
    migrator.do_migration()
    migrator.initialize_samples()


def admin_login_required(method):
    @functools.wraps(method)
    def wrapper(*args, **kwargs):
        if not current_user.is_authenticated:
            flash("This section is for logged in users only.", 'warning')
            return redirect(url_for('redberry.home'))

        if not hasattr(current_user, 'is_admin'):
            flash("Redberry expects your user instance to implement an `is_admin()` method.", 'warning')
            return redirect(url_for('redberry.home'))

        if not current_user.is_admin():
            flash("This section is for admin users only.", 'warning')
            return redirect(url_for('redberry.home'))

        return method(*args, **kwargs)

    return wrapper


############
# CMS ROUTES
############
@cms.route('/')
@cms.route('.amp/', endpoint='home_amp')
def home():
    from redberry.models import RedPost
    posts = RedPost.all_published()
    return render_redberry('redberry/index.html', posts=posts)


@cms.route('/<slug>')
@cms.route('/<slug>.amp', endpoint='show_post_amp')
def show_post(slug):
    from redberry.models import RedPost
    post = RedPost.query.filter_by(slug=slug).first()
    if not post:
        flash("Post not found!", 'danger')
        return redirect(url_for('redberry.home'))

    return render_redberry('redberry/post.html', post=post)


@cms.route('/category/<category_slug>')
@cms.route('/category/<category_slug>.amp', endpoint='show_category_amp')
def show_category(category_slug):
    from redberry.models import RedCategory
    category = RedCategory.query.filter_by(slug=category_slug).first()
    if not category:
        flash("Category not found!", 'danger')
        return redirect(url_for('redberry.home'))
    return render_redberry('redberry/category.html', category=category)


@cms.route('/sitemap')
def build_sitemap():
    from redberry.models import RedPost, RedCategory
    from apesmit import Sitemap
    sm = Sitemap(changefreq='weekly')

    for post in RedPost.all_published():
        sm.add(url_for('redberry.show_post', slug=post.slug, _external=True), lastmod=post.updated_at.date())

    for category in RedCategory.query.all():
        sm.add(url_for('redberry.show_category', category_slug=category.slug, _external=True), lastmod=category.updated_at.date())

    with open(os.path.join(REDBERRY_ROOT, 'static', 'redberry', 'sitemap.xml'), 'w') as f:
        sm.write(f)

    flash("Sitemap created.", 'success')
    return redirect(url_for('redberry.home'))


##############
# ADMIN ROUTES
##############
@cms.route('/admin', methods=['GET'], defaults={'model_name': 'category'})
@cms.route('/admin/<string:model_name>', methods=['GET'])
@admin_login_required
def admin(model_name):
    from redberry.models import RedCategory, RedPost

    if model_name == 'category':
        objects = RedCategory.query.all()

    elif model_name == 'post':
        objects = RedPost.query.all()

    return render_template('redberry/admin/index.html', objects=objects, model_name=model_name)


@cms.route('/admin/<string:model_name>/new', methods=['GET', 'POST'])
@admin_login_required
def new_record(model_name):
    from redberry.models import RedCategory, RedPost
    from redberry.forms import CategoryForm, PostForm

    if model_name == 'category':
        form = CategoryForm()
        new_record = RedCategory()

    elif model_name == 'post':
        form = PostForm()
        new_record = RedPost()

        # Convert category ids into objects for saving in the relationship.
        if form.categories.data:
            form.categories.data = RedCategory.query.filter(RedCategory.id.in_(form.categories.data)).all()
            form.categories.choices = [(c, c.title) for c in RedCategory.sorted()]
        else:
            form.categories.choices = [(c.id, c.title) for c in RedCategory.sorted()]

    if form.validate_on_submit():
        form.populate_obj(new_record)

        cms.config['db'].session.add(new_record)
        cms.config['db'].session.flush()

        build_sitemap()

        flash("Saved %s %s" % (model_name, new_record.id), 'success')
        return redirect(url_for('redberry.admin', model_name=model_name))

    return render_template('redberry/admin/form.html', form=form, model_name=model_name)


@cms.route('/admin/<string:model_name>/<string:slug>', methods=['GET', 'POST'])
@admin_login_required
def edit_record(model_name, slug):
    from redberry.models import RedCategory, RedPost
    from redberry.forms import CategoryForm, PostForm

    if model_name == 'category':
        record = RedCategory.query.filter_by(slug=slug).first()
        form = CategoryForm(obj=record)

    elif model_name == 'post':
        record = RedPost.query.filter_by(slug=slug).first()
        form = PostForm(obj=record)

        # Convert category ids into objects for saving in the relationship.
        form.categories.choices = [(c.id, c.title) for c in RedCategory.sorted()]
        if record.categories and request.method == 'GET':
            form.categories.data = [c.id for c in record.categories]

    if not record:
        flash("%s not found!" % model_name.title(), 'danger')
        return redirect(url_for('redberry.admin'))

    if form.validate_on_submit():

        if model_name == 'post' and form.categories.data:
            form.categories.data = RedCategory.query.filter(RedCategory.id.in_(form.categories.data)).all()

        form.populate_obj(record)
        cms.config['db'].session.flush()
        build_sitemap()
        flash("Saved %s %s" % (model_name, record.id), 'success')
        return redirect(url_for('redberry.admin', model_name=model_name))

    elif request.values.get('_method') == 'DELETE':
        record.delete()
        build_sitemap()
        flash("Deleted %s" % model_name, 'success')
        return redirect(url_for('redberry.admin', model_name=model_name))

    return render_template('redberry/admin/form.html', form=form, object=record, model_name=model_name)


##############
# ADMIN ROUTES
##############
def render_redberry(template_name, **kwargs):

    # For Accelerated Mobile Pages (AMP, ref: https://www.ampproject.org) use templates with a .amp suffix
    if '.amp' in request.path:
        kwargs['render_amp'] = True

    return render_template(template_name, **kwargs)


import logging


def init_logger():
    formatter = logging.Formatter('%(asctime)s %(levelname)s: %(message)s [in %(pathname)s:%(lineno)d]')

    logger = logging.getLogger('redberry')

    logger.setLevel(logging.DEBUG)

    console = logging.StreamHandler()
    console.setFormatter(formatter)
    logger.addHandler(console)



import os

from redberry.models import db, RedModel
from redberry.blueprint import REDBERRY_ROOT


class RedVersion(RedModel):
    __tablename__ = 'redberry_versions'
    id = db.Column(db.Integer, primary_key=True)
    version = db.Column(db.String(255))
    created_at = db.Column(db.DateTime, server_default=db.func.current_timestamp())

    @classmethod
    def last(cls):
        return cls.query.order_by('created_at DESC').first()

    @classmethod
    def all_migrations(cls):
        migrations = []

        files = os.listdir(os.path.join(REDBERRY_ROOT, 'models', 'migrations'))
        for f in files:
            try:
                bits = filter(lambda x: x is not None, f.split('_'))
                if len(bits) > 1 and int(bits[0]):
                    migrations.append(f)
            except:
                pass

        return migrations

    @classmethod
    def already_run(cls, migration_file):
        return cls.query.filter_by(version=migration_file).first()

    @classmethod
    def store_migration(cls, migration_file):
        migration = RedVersion(version=migration_file)
        db.session.add(migration)
        db.session.flush()





import logging
from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask.ext.testing import TestCase
from flask.ext.login import LoginManager, AnonymousUserMixin


app = Flask(__name__)
db = SQLAlchemy(app, session_options={'autocommit': True, 'expire_on_commit': False})

login_manager = LoginManager()
login_manager.init_app(app)
login_manager.anonymous_user = AnonymousUserMixin

url_prefix = 'blog_test'

from redberry.blueprint import cms
app.config['redberry.config'] = {'db': db}
app.register_blueprint(cms, url_prefix="/%s" % url_prefix)


class RedTestCase(TestCase):
    """
    Custom test case class for any common methods.
    """
    def __init__(self, methodName):
        super(RedTestCase, self).__init__(methodName)
        self.test_client = app.test_client()
        self.db = db
        self.logger = logging.getLogger('redberry.tests')
        self.url_prefix = url_prefix

    def create_app(self):
        app.config['SECRET_KEY'] = '8cauliflowers!'
        app.config['WTF_CSRF_ENABLED'] = False
        app.config['PROPAGATE_EXCEPTIONS'] = True
        return app

    def setUp(self):
        from redberry.models.migrate import RedMigrator
        migrator = RedMigrator()
        migrator.initialize_samples()

    def tearDown(self):
        from redberry.models import RedCategory, RedPost
        RedCategory.query.delete()
        RedPost.query.delete()
        self.db.session.execute("DELETE FROM redberry_categories_posts")

    def assert_flashes(self, expected_message):
        with self.test_client.session_transaction() as session:
            try:
                message_tuples = session['_flashes']
                messages = [t[1] for t in message_tuples]
            except KeyError:
                raise AssertionError('nothing flashed')
            assert expected_message in messages



from sqlalchemy import event

from redberry.models import db, RedModel, generate_slug


class RedCategory(RedModel):
    __tablename__ = 'redberry_categories'
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(255))
    slug = db.Column(db.String(255))
    description = db.Column(db.Text)
    created_at = db.Column(db.DateTime, server_default=db.func.current_timestamp())
    updated_at = db.Column(db.DateTime, server_default=db.func.current_timestamp(), onupdate=db.func.current_timestamp())

    @classmethod
    def sorted(cls):
        return cls.query.order_by(cls.title).all()


event.listen(RedCategory.title, 'set', generate_slug, retval=False)


from flask_wtf import FlaskForm
from wtforms import validators, StringField, TextAreaField


class CategoryForm(FlaskForm):
    title = StringField('Title', [validators.DataRequired()])
    description = TextAreaField('Description')



import collections
from sqlalchemy import event

from redberry.models import db, RedModel, generate_slug


categories_posts = db.Table('redberry_categories_posts',
                              db.Column('category_id', db.Integer, db.ForeignKey('redberry_categories.id'), nullable=False),
                              db.Column('post_id', db.Integer, db.ForeignKey('redberry_posts.id'), nullable=False),
                              db.PrimaryKeyConstraint('category_id', 'post_id'))


class RedPost(RedModel):
    __tablename__ = 'redberry_posts'
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(255))
    slug = db.Column(db.String(255))
    content = db.Column(db.Text)
    summary = db.Column(db.Text)
    published = db.Column(db.Boolean, default=True, index=True)
    created_at = db.Column(db.DateTime, server_default=db.func.current_timestamp())
    updated_at = db.Column(db.DateTime, server_default=db.func.current_timestamp(), onupdate=db.func.current_timestamp())

    categories = db.relationship("RedCategory", secondary=categories_posts, backref='posts')

    @classmethod
    def all_published(cls):
        return cls.query.filter_by(published=True).order_by('created_at DESC').all()

    def keywords(self, num=5):

        words_only = self.strip_tags(self.content, strip_punctuation=True)
        words = words_only.split()

        counter = collections.Counter(words)
        common = counter.most_common()

        keywords = []
        UNIMPORTANT_WORDS = ('a', 'the', 'as', 'be', 'if', 'then', 'when', 'why', 'what', 'so', 'it', 'to', 'just', 'but')

        for word in common:
            if word[0].lower() not in UNIMPORTANT_WORDS:
                keywords.append(word[0].lower())

            if len(keywords) >= num:
                break

        return ", ".join(keywords)


event.listen(RedPost.title, 'set', generate_slug, retval=False)



from flask_wtf import FlaskForm
from wtforms import validators, StringField, BooleanField, TextAreaField, SelectMultipleField


class PostForm(FlaskForm):
    title = StringField('Title', [validators.DataRequired()])
    summary = TextAreaField('Summary')
    content = TextAreaField('Content')
    categories = SelectMultipleField('Categories', coerce=int)
    published = BooleanField('Published')



from redberry.tests import RedTestCase
from redberry.models import RedPost, RedCategory


class BlueprintTest(RedTestCase):

    def test_init_redberry_creates_samples(self):
        assert RedPost.query.count() >= 1
        assert RedCategory.query.count() >= 1



from redberry.tests import RedTestCase
from redberry.models import RedPost, RedCategory


class FrontendRoutesTest(RedTestCase):

    def test_home(self):
        url = '/%s/' % self.url_prefix
        response = self.test_client.get(url)

        # Shows published
        post = RedPost.query.first()
        assert post.title in response.data
        assert post.content in response.data

        # Does not show if not published
        post.published = False
        response = self.test_client.get(url)
        assert post.title not in response.data
        assert post.content not in response.data
        assert "No posts yet." in response.data

    def test_show_post(self):
        post = RedPost.query.first()
        url = "/%s/%s" % (self.url_prefix, post.slug)

        # Shows by slug
        response = self.test_client.get(url)
        assert post.title in response.data
        assert post.content in response.data

        # Redirects if invalid slug
        response = self.test_client.get(url + "-invalid")
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/' % self.url_prefix
        self.assert_flashes("Post not found!")

    def test_show_category(self):
        category = RedCategory.query.first()
        url = "/%s/category/%s" % (self.url_prefix, category.slug)

        # Shows by slug
        response = self.test_client.get(url)
        assert category.title in response.data
        assert category.description in response.data

        # Shows category posts
        if category.posts:
            assert category.posts[0].title in response.data

        # Redirects if invalid slug
        response = self.test_client.get(url + "-invalid")
        assert response.status_code == 302
        assert response.location == 'http://localhost/%s/' % self.url_prefix
        self.assert_flashes("Category not found!")




import os
from setuptools import setup, find_packages

# Dynamically parse version number from VERSION file
direc = os.path.dirname(os.path.realpath(__file__))
version = open(direc + os.sep + 'VERSION').readline().strip()


# Dynamically parse install requirements from requirements.txt
def parse_requirements(test_requirements=False):
    packages = []

    if test_requirements:
        requirements_file = open(os.path.join(direc, 'redberry', 'tests', 'requirements.txt'))
    else:
        requirements_file = open(os.path.join(direc, 'requirements.txt'))

    for line in requirements_file:
        package = line.strip()

        if not package:
            break

        packages.append(package)

    return packages

requirements = parse_requirements(test_requirements=False)
test_requirements = parse_requirements(test_requirements=True)

setup(
    name='Redberry',
    version=version,
    description='Flask Blueprint for adding simple CMS functionality',
    long_description='Flask Blueprint for adding simple CMS functionality',
    author='Michael Cho',
    author_email='michael.cho@mail.com',
    license='Apache License 2.0',
    url='https://github.com/michaelcho/redberry',

    install_requires=requirements,
    tests_require=test_requirements,
    packages=find_packages(),

    # Includes templates and static files in MANIFEST.in
    include_package_data=True,

    # Ref https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Flask',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='flask cms blog blueprint',
)
